/*
 *
 * CLIF is a Load Injection Framework
 * Copyright (C) 2004, 2005, 2006 France Telecom
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * CLIF
 *
 * Contact: clif@objectweb.org
 */
package org.objectweb.clif.isac.plugins.httpinjector10.tools;

import java.util.Calendar;
import java.util.Date;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.util.Map;

import org.apache.commons.httpclient.Cookie;
import org.apache.commons.httpclient.HttpState;
import org.apache.commons.httpclient.UsernamePasswordCredentials;
import org.apache.commons.httpclient.auth.AuthScope;
import org.apache.commons.httpclient.cookie.CookiePolicy;
import org.objectweb.clif.isac.plugins.httpinjector10.SessionObject;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.wrapper.javaLog.LoggerFactory;

/**
 * Class that deals with the HttpState of a connection Created on august 17 2004
 *
 * @author JC Meillaud
 * @author A Peyrard
 * @author Bruno Dillenseger
 */
public class HttpStateUtils {

	// logger
	static protected Logger log = new LoggerFactory().getLogger(HttpStateUtils.class.getName()); 
	/**
	 * Function which return the CookiePolicy (an int) corresponding to the
	 * String cookiePolicy
	 *
	 * @param cookiePolicy
	 *            String representing what sort of policy
	 * @return the CookiePolicy corresponding
	 */
	private static String returnCookiePolicy(String cookiePolicy)
	{
		if (cookiePolicy.equals(ParameterConstants.RFC2109))
		{
			return CookiePolicy.RFC_2109;
		}
		else if (cookiePolicy.equals(ParameterConstants.NETSCAPE))
		{
			return CookiePolicy.NETSCAPE;
		}
		else if (cookiePolicy.equals(ParameterConstants.NOCOOKIE))
		{
			return CookiePolicy.IGNORE_COOKIES;
		}
		else
		{
			return CookiePolicy.BROWSER_COMPATIBILITY;
		}
	}

	/**
	 * Method to deserialize the cookie parameters and build and hachtable from
	 * it
	 *
	 * @param inputParameters
	 *            the serialized String representing the cookie
	 * @return an Hachtable containing all the variable
	 */
	private static Hashtable setCookieParameters(String inputParameters) {
		log.log(BasicLevel.DEBUG, "-> [Enter] setCookieParameters");
		Hashtable result = new Hashtable();
		StringTokenizer st = new StringTokenizer(inputParameters, "|");
		while (st.hasMoreTokens()) {
			String inputParameter = st.nextToken();
			log.log(BasicLevel.DEBUG, "element : " + inputParameter);
			StringTokenizer st2 = new StringTokenizer(inputParameter, "=");
			if (st2.countTokens() == 2) {
				String inputParamName = st2.nextToken();
				String inputParamValue = st2.nextToken();
				log.log(BasicLevel.DEBUG, "CookieParamName : ["
						+ inputParamName + "] ;CookieParamValue : ["
						+ inputParamValue + "]");
				if (!inputParamName.equals("") && !inputParamValue.equals(""))
					result.put(inputParamName, inputParamValue);
				else
					log
							.log(BasicLevel.DEBUG,
									"One of the parameters for the cookie is undefined");
			}
		}
		log.log(BasicLevel.DEBUG, "<- [Exit] setCookieParameters");
		return result;
	}

	/**
	 * Method wich return a httpclient cookie described into the input hashtable
	 *
	 * @param params
	 *            the hashtable describing the cookie
	 * @return an HttpClient Cookie
	 */
	private static Cookie setCookie(Hashtable params) {
		log.log(BasicLevel.DEBUG, "-> [Enter] setCookie");
		Cookie cookie = new Cookie();
		String domain = (String) params.get(ParameterConstants.DOMAIN);
		String name = (String) params.get(ParameterConstants.NAME);
		String value = (String) params.get(ParameterConstants.VALUE);
		String path = (String) params.get(ParameterConstants.PATH);
		String expires = (String) params.get(ParameterConstants.EXPIRES);
		String maxAge = (String) params.get(ParameterConstants.MAXAGE);
		String secure = (String) params.get(ParameterConstants.SECURE);
		secure = (secure != null) ? secure : "";
		if (expires != null && !expires.equals("")) {
			StringTokenizer st = new StringTokenizer(expires, "/");
			Calendar expiresDate = null;
			if (st.countTokens() == 5) {
				// year / month / date / hour / minute / seconde
				expiresDate.setTime(new Date());
				expiresDate.set(new Integer(st.nextToken()).intValue(),
						new Integer(st.nextToken()).intValue(), new Integer(st
								.nextToken()).intValue(), new Integer(st
								.nextToken()).intValue(), new Integer(st
								.nextToken()).intValue(), new Integer(st
								.nextToken()).intValue());
			} else {
				log
						.log(BasicLevel.DEBUG,
								"There is a problem in the definition of the expire Date of the cookie");
			}
			cookie = new Cookie(domain, name, value, path, expiresDate
					.getTime(), new Boolean(secure).booleanValue());
		} else if (maxAge != null && !maxAge.equals("")) {
			cookie = new Cookie(domain, name, value, path, new Integer(maxAge)
					.intValue(), new Boolean(secure).booleanValue());
		} else {
			cookie = new Cookie(domain, name, value);
		}
		cookie
				.setDomainAttributeSpecified(domain != null
						&& !domain.equals(""));
		cookie.setPathAttributeSpecified(path != null && !path.equals(""));

		log.log(BasicLevel.DEBUG, "<- [Exit] setCookie");
		return cookie;
	}

	/**
	 * build an httpState that is a container for HTTP attributes that may
	 * persist from request to request, such as cookies and authentication
	 * credentials.
	 *
	 * @param params
	 *            Hashtable containing all the variable
	 * @return an HttpState containing all Http attributes
	 */
	public static HttpState setHttpState(SessionObject sessionObject,
			Map params) {
		log.log(BasicLevel.DEBUG, "-> [Enter] setHttpState");
		HttpState httpState = (sessionObject.getHttpClient().getState() != null)
				? sessionObject.getHttpClient().getState()
				: new HttpState();
		// Firstly configure the credentials if they are defined
		if (params.containsKey(ParameterConstants.PROXYUSERNAME)) {
			String proxyLogin = (String) params
				.get(ParameterConstants.PROXYUSERNAME);
			String proxyPassword = (String) params
				.get(ParameterConstants.PROXYUSERPASS);
			if (proxyLogin != null
				&& proxyLogin != ""
				&& proxyPassword != null
				&& proxyPassword != "")
			{
				httpState.setProxyCredentials(
					new AuthScope(AuthScope.ANY_HOST, AuthScope.ANY_PORT),
					new UsernamePasswordCredentials(proxyLogin, proxyPassword));
			}
		}

		// Secondly build the cookies defined and add them to the HttpState
		if (params.containsKey(ParameterConstants.COOKIES)) {

			StringTokenizer st = new StringTokenizer((String) params
					.get(ParameterConstants.COOKIES), ";");
			while (st.hasMoreTokens()) {
				String cookieParameters = st.nextToken();
				httpState
						.addCookie(setCookie(setCookieParameters(cookieParameters)));
			}
		}
		// Lastly configure or reconfigure the general policy for the cookies
		if (params.containsKey(ParameterConstants.COOKIEPOLICY))
		{
			sessionObject.getHttpClient().getParams().setCookiePolicy(
				returnCookiePolicy((String) params.get(ParameterConstants.COOKIEPOLICY)));
		}
		log.log(BasicLevel.DEBUG, "<- [Exit] setHttpState");
		return httpState;

	}

	// DEBUG Method

	/**
	 * Method to print the Cookies which are in the SessionObject
	 *
	 * @param sessionObject
	 *            The SessionObject
	 */
	public static void printCookies(SessionObject sessionObject) {
		Cookie[] cookies = sessionObject.getHttpClient().getState()
				.getCookies();
		log.log(BasicLevel.DEBUG, "Cookie Present : " + cookies.length);
		for (int i = 0; i < cookies.length; i++) {
			log.log(BasicLevel.DEBUG, " - " + cookies[i].toExternalForm());
		}
	}

}